#' Add external Resources to the Application
#'
#' This function is internally used to add external
#' resources inside the Shiny application.
#'
#' @import shiny
#' @importFrom golem add_resource_path activate_js favicon bundle_resources
#' @noRd
golem_add_external_resources <- function() {
  add_resource_path(
    "www",
    app_sys("app/www")
  )
  tags$head(
    golem::activate_js(),
    golem::favicon(),
    bundle_resources(
      path = app_sys("app/www"),
      app_title = "analycyte"
    )
  )
}


#' The application User-Interface
#'
#' @param request Internal parameter for `{shiny}`.
#'     DO NOT REMOVE.
#' @import shiny
#' @import shinydashboard
#' @importFrom shinyjs useShinyjs
#' @importFrom shinyWidgets actionBttn
#' @noRd
app_ui <- function(request) {
  tagList(
    golem_add_external_resources(),
    shinyjs::useShinyjs(),
    dashboardPage(
      dashboardHeader(
        title = tags$a("analycyte", href = "", class = "title"),
        tags$li(
          id = "historyToggle",
          class = "historyLabel dropdown tasks-menu",
          tags$span(id = "historyArrow", class = "fas", icon("angle-right")), HTML("&nbsp;"),
          "history"
        ),
        tags$li(
          actionButton("close_app", "",
            icon("power-off"),
            class = "close-my-app-btn btn-danger"
          ),
          class = "dropdown tasks-menu"
        )
      ),
      dashboardSidebar(
        sidebarMenu(
          id = "sidebarid",
          uiOutput("projectSelect"),
          uiOutput("sceSelect"),
          uiOutput("feSelect"),
          menuItem(
            "Dashboard",
            tabName = "entree", icon = icon("dashboard"), selected = T
          ),
          menuItemOutput("tab_dynamic"),
          menuItemOutput("data_dynamic"),
          menuItem("Session", tabName = "in_session", icon = icon("sync")),
          uiOutput("projectSetupUI"),
          actionButton("reload", "", class = "btn-dir reload", icon = icon("arrows-rotate"))
        )
      ),
      dashboardBody(
        div(
          id = "rightSidebar", class = "inactive",
          uiOutput("steps_table")
        ),
        tabItems(
          tabItem(
            tabName = "entree",
            fluidRow(
              id = "show_tool_box",
              box(
                width = 3, uiOutput("toolSelect"), status = "primary"
                # , uiOutput("insert_box_launch")
              ),
            ),
            tags$br(),
            tabsetPanel(
              id = "myTabs", type = "tabs"
            )
          ),
          tabItem(
            tabName = "project",
            fluidRow(box(width = 12, title = "Project", mod_table_ui("project_table")))
          ),
          tabItem(
            tabName = "authors_affiliations",
            fluidRow(
              box(width = 6, title = "Authors", mod_table_ui("author_table")),
              box(width = 6, title = "Affiliations", mod_table_ui("affilation_table"))
            ),
            fluidRow(
              datamods::edit_data_ui("author_bis")
            )
          ),
          tabItem(
            tabName = "tools",
            fluidRow(box(width = 12, title = "Tools", uiOutput("tool_table")))
          ),
          tabItem(
            tabName = "history",
            fluidRow(box(
              width = 12, title = "History",
              mod_table_ui("large_steps_table", show_add_button = FALSE)
            ))
          ),
          tabItem(
            tabName = "in_data",
            mod_dataset_ui("current_fe")
          ),
          tabItem(
            tabName = "in_session",
            fluidRow(
              box(
                status = "primary", width = 12,
                uiOutput("env_values"),
                uiOutput("env_export_project")
              )
            )
          )
        )
      )
    )
  )
}
