#' qc UI Function
#'
#' @description A shiny Module.
#'
#' @param id,input,output,session Internal parameters for {shiny}.
#'
#' @noRd
#' @import bsplus
#' @importFrom shiny NS tagList
#' @importFrom shinyWidgets pickerOptions actionBttn
#' @importFrom shinycssloaders withSpinner
mod_da_ui <- function(id, r_global, type = NULL) {
  ns <- NS(id)
  tagList(
    div(
      id = id,
      fluidRow(
        box(
          width = 12, status = "primary",
          div(
            column(
              width = 6, strong("Project of Reference: "),
              span(style = "color: #007bff;", r_global$project_folder_name) # Styling for the text
            ),
            column(
              width = 6, strong("Folder of Reference: "),
              span(style = "color: #007bff;", r_global$dataFE) # Styling for the text
            )
          )
        )
      ),
      fluidRow(
        column(
          width = 5,
          uiOutput(ns("dynamic_h2"))
        ),
        column(width = 4),
        column(
          width = 3,
          class = "run_report",
          actionBttn(
            inputId = ns("run_diff"),
            class = "btn-secondary",
            icon = icon("gears"),
            style = "material-flat",
            label = "Run the report"
          )
        )
      ), # h2("Differential Abundance"),

      fluidRow(
        column(
          width = 6, uiOutput(ns("dynamic_radio_buttons"))
          # radioButtons(ns("analyse_type"), "Analyse Type:", choices = c(
          #   "Differential Abundance" = "DA",
          #   "Debug" = "DB"
          # ), selected = "DA", inline = T)
        )



        # box(width = 6,
        #     "path"
        # ),
        #
        # box(width = 6,
        #     "project"
        # ),
        # box(width = 6,
        #     "rds"
        # ),
      ),
      fluidRow(
        tabBox(
          height = 300,
          width = 7, side = "left",
          title = "",
          selected = "General Parameters",
          id = "param_set",
          tabPanel("TRUE/FALSE",
            id = "yes_no_set",
            tagList(
              fluidRow(
                column(
                  width = 12,
                  checkboxInput(ns("codex"), "Add code Info", value = FALSE),
                  checkboxInput(ns("logex"), "Log on graphs", value = FALSE),
                  checkboxInput(ns("sortex"), "Sort on graphs", value = FALSE),
                  checkboxInput(ns("show_param_yaml"), "Show the parameter table", value = TRUE),
                  checkboxInput(ns("show_summary_data"), "Show the summary tables", value = TRUE)
                )
              )
            )
          ),
          tabPanel("Tresholds",
            id = "thresh_set",
            tagList(
              fluidRow(
                column(
                  width = 6,
                  uiOutput(ns("o_min_cells"))
                ),
                column(
                  width = 6,
                  uiOutput(ns("o_min_samples"))
                ),
                column(
                  width = 6,
                  sliderInput(ns("choose_fc"), "Fold Change threshold:",
                    min = 0,
                    max = 10,
                    value = 1.0,
                    step = 0.1
                  )

                  # uiOutput(ns("fixed_fc"))
                ),
                column(
                  width = 6,
                  div(class = "InputSized", numericInput(ns("given_pvalue"), "pvalue cutoff:", 0.001, min = 0, max = 1, step = 0.001))
                )
              )
            )
          ),
          tabPanel("General Parameters",
            id = "select_target_set",
            tagList(
              fluidRow(
                column(
                  width = 6,
                  uiOutput(ns("conditions_"))
                ),
                column(
                  width = 6,
                  div(class = "InputSized", uiOutput(ns("batch_")))
                ),

                # column(
                #   width = 4, div(class = "InputSized", uiOutput(ns("cluster_id_")))
                # ),
                # column(
                #   width = 4,
                #   div(class = "InputSized", uiOutput(ns("ncells_")))
                # ),
                column(
                  width = 6,
                  uiOutput(ns("annot_on_plots_"))
                ),
                column(
                  width = 6, div(class = "InputSized", uiOutput(ns("annotation_metacluster")))
                )
                # ,box(
                #   width = 6,
                #   "subset_column"
                # )
              )
            )
          ),
          tabPanel("Univariate",
            id = "univariate_set",
            tagList(
              fluidRow(
                column(
                  width = 6,
                  div(uiOutput(ns("design_")))
                ),
                column(
                  width = 6,
                  div(class = "InputSized", uiOutput(ns("block_id_")))
                ),
                column(
                  width = 6
                  # ,  div(uiOutput(ns("design_GLMM_fixed_")))
                ),
                column(
                  width = 6,
                  div(uiOutput(ns("design_GLMM_random_")))
                )
              ),
              fluidRow(
                column(width = 6, uiOutput(ns('reference_'))),
                column(width = 6, uiOutput(ns('contrasts_')))
              ),
              fluidRow(column(
                width = 4,
                checkboxInput(ns("transform"), "Transform data", value = FALSE)
              ))
            )
          )
        ),
        uiOutput(ns("box_here"))
      ),
      fluidRow(
        column(
          width = 8,
          withSpinner(DT::dataTableOutput(ns("metadata_FE_running")), type=6)
        ),
        column(
          width = 4,
          withSpinner(DT::dataTableOutput(ns("rowData_FE_running")), type=6)
        )
      )
    )
  )
}

#' qc Server Functions
#' @import bsplus
#' @importFrom shiny NS tagList
#' @importFrom shinyWidgets pickerOptions
#' @noRd
mod_da_server <- function(id, r_global, reloadTrigger, type = NULL) {
  moduleServer(id, function(input, output, session) {
    ns <- session$ns

    output$box_here <- renderUI({
      box(
        width = 5,
        status = "warning", solidHeader = T, collapsible = T, collapsed = F,
        title = "Differential module",
        markdown('
**Functionality Overview:**
The Differential Analysis Panel is designed for users to conduct comprehensive differential analyses. It includes advanced settings for thresholds, like p-value and fold change, and a dedicated tab for setting univariate parameters to define the differential analysis design.

**Instructions:**

1. **General Parameters:**
   -  Define the Condition column on which the differential comparison is to be performed.
1. **Threshold Settings:**
   - In the \'Thresholds\' tabset, set critical values for your analysis.
   - This includes specifying a p-value threshold for statistical significance and a fold change threshold to identify meaningful differences.
2. **Univariate Parameters:**
   - Navigate to the new \'Univariate Parameters\' tab to configure the design of your differential analysis. You will need to confirm the condition the \'Design columns\' selector.
3. **Run Analysis:**
   - After setting all thresholds and univariate parameters, initiate the differential analysis by clicking the "RUN ANALYSIS" button.

**Additional Information:**
- Please ensure that your browser allows pop-up windows to be opened.

')
      )
    })

    dataFE <- reactiveVal()
    # curent hash r_global$dataFE
    dataFE(r_global$data)
    # print(type)
    # Dynamic H2 Tag
    output$dynamic_h2 <- renderUI({
      if (type == "mod_da") {
        h2("Differential Abundance", class = "module-title-bold")
      } else if (type == "mod_ds") {
        h2("Differential State", class = "module-title-bold")
      } else {
        h2("Please check module name")
      }
    })

    # Dynamic Radio Buttons
    output$dynamic_radio_buttons <- renderUI({
      if (type == "mod_da") { # Replace 'some_condition' with your specific condition
        column(
          width = 12,
          radioButtons(ns("analyse_type"), "Analyse Type:", choices = c(
            "Differential Abundance" = "DA",
            "Differential Abundance Extended" = "DA_ext",
            "Control" = "CT"
          ), selected = "DA", inline = TRUE)
        )
      } else {
        column(
          width = 6,
          radioButtons(ns("analyse_type"), "Analyse Type:", choices = c(
            "Differential State" = "DS",
            "Control" = "CT"
          ), selected = "DS", inline = TRUE)
        )
      }
    })


    meta_names <- reactiveVal()



    fcs_data <- reactive({
      req(dataFE())
      getFeature(dataFE()$SE_abundance, c("colData")) |> as.data.frame()
    })
    row_data <- reactive({
      req(dataFE())
      getFeature(dataFE()$SE_abundance, c("rowData")) |> as.data.frame()
    })

    output$metadata_FE_running <- DT::renderDataTable(
      {
        prettyDT(fcs_data(), selection = "multiple")
      },
      server = F
    )

    output$rowData_FE_running <- DT::renderDataTable(
      {
        prettyDT(row_data(), selection = "multiple", rownames = FALSE)
      },
      server = F
    )




    observe({
      req(dataFE())
      names <- analycyte.utils::getFeature(dataFE()$SE_abundance, c("colData"))
      meta_names(names)
    })
    col_names <- reactiveVal()

    observe({
      req(dataFE())
      names <- analycyte.utils::getFeature(dataFE()$SE_abundance, c("rowData"))
      col_names(names)
    })

    output$o_min_cells <- renderUI({
      req(dataFE())

      sliderInput(ns("min_cells"), "Minimum cells treshold:",
        min = 0,
        max = 500,
        value = 10,
        step = 1
      )
    })


    output$o_min_samples <- renderUI({
      req(dataFE())
      n_sample <- ncol(dataFE()$SE_abundance)
      sliderInput(ns("min_samples"), "Minimum samples:",
        min = 0,
        max = n_sample,
        value = round(n_sample / 4),
        step = 1
      )
    })


    output$conditions_ <- renderUI({
      req(dataFE())
      pickerInput(
        inputId = ns("selected_conditions"), "Condition:", choices = colnames(meta_names()), selected = NULL,
        multiple = TRUE,
        options = pickerOptions(maxOptions = 1)
      ) |>
        shinyInput_label_embed(
          shiny_iconlink() |>
            bs_embed_popover(
              title = "Column with the condition of interest, conditions parameter in yaml header", content = "Choose a column name with 2 parameters", placement = "left"
            )
        )
    })

    output$batch_ <- renderUI({
      req(dataFE())

      pickerInput(
        inputId = ns("selected_batch"), "Batch Column:", choices = colnames(meta_names()), selected = NULL,
        multiple = TRUE,
        options = pickerOptions(maxOptions = 1)
      ) |>
        shinyInput_label_embed(
          shiny_iconlink() |>
            bs_embed_popover(
              title = "Column with the possible Batch parameter, batch parameter in yaml header", content = "Choose a column name with a batch parameter", placement = "left"
            )
        )
    })


    output$annotation_metacluster <- renderUI({
      req(dataFE())
      req(row_data())
      pickerInput(
        inputId = ns("selected_annotation"), "Annotation Column:", choices = row_data() |> colnames(),
        selected = NULL,
        multiple = TRUE,
        options = pickerOptions(maxOptions = 1)
      ) |>
        shinyInput_label_embed(
          shiny_iconlink() |>
            bs_embed_popover(
              title = "Column with the possible Annotation parameter, annotation_column parameter in yaml header", content = "Choose a column name with an annotation reference", placement = "left"
            )
        )
    })

    # output$cluster_id_ <- renderUI({
    #   req(dataFE())
    #
    #   pickerInput(
    #     inputId = ns("selected_cluster_id_"), "Cluster ref Column:", choices = row_data() |> colnames(),
    #     selected = 1,
    #     multiple = FALSE
    #   ) |>
    #     shinyInput_label_embed(
    #       shiny_iconlink() |>
    #         bs_embed_popover(
    #           title = "Column with the cluster reference, cluster_id parameter in yaml header", content = "Choose the column with clustering reference", placement = "left"
    #         )
    #     )
    # })

    # output$individuals_ <- renderUI({
    #   req(dataFE())
    #
    #   pickerInput(
    #     inputId = ns("selected_individuals"), "Sample ID:", choices = colnames(meta_names()), selected = NULL,
    #     multiple = TRUE,
    #     options = pickerOptions(maxOptions = 1)
    #   ) |>
    #     shinyInput_label_embed(
    #       shiny_iconlink() |>
    #         bs_embed_popover(
    #           title = "Column with the with the Sample ID, individuals parameter in yaml header", content = "Choose a column name with the sample id", placement = "left"
    #         )
    #     )
    # })


    # output$ncells_ <- renderUI({
    #   req(dataFE())
    #
    #   pickerInput(
    #     inputId = ns("selected_ncells"), "Number of cells:", choices = colnames(meta_names()), selected = NULL,
    #     multiple = TRUE,
    #     options = pickerOptions(maxOptions = 1)
    #   ) |>
    #     shinyInput_label_embed(
    #       shiny_iconlink() |>
    #         bs_embed_popover(
    #           title = "Column with the number of cells per fcs, ncells parameter in yaml header", content = "Choose a column name", placement = "left"
    #         )
    #     )
    # })



    output$annot_on_plots_ <- renderUI({
      req(dataFE())

      pickerInput(
        inputId = ns("selected_annot_on_plots"), "Meta annotation:", choices = colnames(meta_names()), selected = NULL,
        multiple = TRUE
      ) |>
        shinyInput_label_embed(
          shiny_iconlink() |>
            bs_embed_popover(
              title = "Columns to use for the graphics annotation, plot_legends parameter in yaml header", content = "Choose columns to annotate the figures", placement = "left"
            )
        )
    })



    output$block_id_ <- renderUI({
      req(dataFE())
      pickerInput(
        inputId = ns("selected_block_id"), "Block ID:",
        choices = colnames(meta_names()), selected = NULL,
        multiple = TRUE,
        options = pickerOptions(maxOptions = 1)
      ) |>
        shinyInput_label_embed(
          shiny_iconlink() |>
            bs_embed_popover(
              title = "(Optional) Vector or factor of block IDs (e.g. patient IDs) for paired experimental designs, to be included as random effects. If provided, the block IDs will be included as random effects using the limma duplicateCorrelation methodology. Alternatively, block IDs can be included as fixed effects in the design matrix (createDesignMatrix).", content = "Choose a column name for fixed effect", placement = "left"
            )
        )
    })


    output$design_ <- renderUI({
      req(dataFE())

      pickerInput(
        inputId = ns("selected_design"), "Design columns:", choices = colnames(meta_names()), selected = NULL,
        multiple = TRUE
      ) |>
        shinyInput_label_embed(
          shiny_iconlink() |>
            bs_embed_popover(
              title = "x", content = "y", placement = "left"
            )
        )
    })
    # output$design_GLMM_fixed_ <- renderUI({
    #   req(dataFE())
    #
    #   pickerInput(
    #     inputId = ns("selected_design_GLMM_fixed"), "Design GLMM fixed:", choices = colnames(meta_names()), selected = NULL,
    #     multiple = TRUE
    #   ) |>
    #     shinyInput_label_embed(
    #       shiny_iconlink() |>
    #         bs_embed_popover(
    #           title = "x", content = "y", placement = "left"
    #         )
    #     )
    # })

    output$design_GLMM_random_ <- renderUI({
      req(dataFE())

      pickerInput(
        inputId = ns("selected_design_GLMM_random"), "Design GLMM random:", choices = colnames(meta_names()), selected = NULL,
        multiple = TRUE
      ) |>
        shinyInput_label_embed(
          shiny_iconlink() |>
            bs_embed_popover(
              title = "TO DO", content = "TO DO", placement = "left"
            )
        )
    })

    

    output$reference_ <- renderUI({
      req(dataFE())
      req(meta_names())
      req(input$selected_conditions)
      
      pickerInput(
        inputId = ns("selected_reference"), "Selected Reference:", choices =unique(meta_names()[[input$selected_conditions]]), selected = NULL,
        multiple = TRUE,
        options = pickerOptions(maxOptions = 1)
      ) |>
        shinyInput_label_embed(
          shiny_iconlink() |>
            bs_embed_popover(
              title = "TO DO", content = "TO DO", placement = "left"
            )
        )
    })
    
    
    output$contrasts_ <- renderUI({
      req(dataFE())
      req(meta_names())
      req(input$selected_conditions)
      req(input$selected_reference)
      
      contrast <- unique(meta_names()[[input$selected_conditions]])
      contrast <- contrast[contrast != input$selected_reference]
      
      pickerInput(
        inputId = ns("selected_contrasts"), "Selected Contrast:", choices = contrast,
        selected = NULL,
        multiple = TRUE
      ) |>
        shinyInput_label_embed(
          shiny_iconlink() |>
            bs_embed_popover(
              title = "TO DO", content = "TO DO", placement = "left"
            )
        )
    })
    
    
    selected_values_string <- reactiveVal(NULL)
    selected_values_design <- reactiveVal(NULL)
    # selected_values_GLMM_fixed <- reactiveVal(NULL)
    selected_values_GLMM_random <- reactiveVal(NULL)
    selected_values_contrasts <- reactiveVal(NULL)
    
    prev_selected <- reactiveVal(NULL)
    prev_design_selected <- reactiveVal(NULL)
    # prev_design_GLMM_fixed_selected <- reactiveVal(NULL)
    prev_design_GLMM_random_selected <- reactiveVal(NULL)
    prev_contrasts_selected <- reactiveVal(NULL)
    

    observe({
      if (!identical(input$selected_annot_on_plots, prev_selected())) {
        prev_selected(input$selected_annot_on_plots)

        if (length(input$selected_annot_on_plots) == 0) {
          selected_values_string(NULL)
        } else {
          selected_values_string(input$selected_annot_on_plots)
        }
      }
    })


    observe({
      if (!identical(input$selected_design, prev_design_selected())) {
        prev_design_selected(input$selected_design)

        if (length(input$selected_design) == 0) {
          selected_values_design(NULL)
        } else {
          selected_values_design(input$selected_design)
        }
      }
    })


    # observe({
    #   if (!identical(input$selected_design_GLMM_fixed, prev_design_GLMM_fixed_selected())) {
    #     prev_design_GLMM_fixed_selected(input$selected_design_GLMM_fixed)
    #
    #     if (length(input$selected_design_GLMM_fixed) == 0) {
    #       selected_values_GLMM_fixed(NULL)
    #     } else {
    #       selected_values_GLMM_fixed(input$selected_design_GLMM_fixed)
    #     }
    #   }
    # })

    observe({
      if (!identical(input$selected_design_GLMM_random, selected_values_GLMM_random())) {
        selected_values_GLMM_random(input$selected_design_GLMM_random)

        if (length(input$selected_design_GLMM_random) == 0) {
          prev_design_GLMM_random_selected(NULL)
        } else {
          prev_design_GLMM_random_selected(input$selected_design_GLMM_random)
        }
      }
    })

    
    observe({
      if (!identical(input$selected_contrasts, selected_values_contrasts())) {
        selected_values_contrasts(input$selected_contrasts)
        
        if (length(input$selected_contrasts) == 0) {
          prev_contrasts_selected(NULL)
        } else {
          prev_contrasts_selected(input$selected_contrasts)
        }
      }
    })
    
    




    observeEvent(
      input$run_diff,
      ignoreInit = TRUE,
      {
        # P_num <- as.numeric(gsub("p|0", "", r_global$project_folder_name))
        # Create a temporary directory
        new_step(
          project = r_global$project_id,
          label = type, output_type = "html", input_hash_tag = r_global$dataFE
        ) -> new_step


        step_row <- tail(new_step, n = 1)
        title <- projects(r_global$project_id) |> dplyr::pull(title)

        # Create the sensory report
        res_info_report <- withProgress(message = "Rendering report...", value = 0, {
          create_report(
            # list_param_report = list(
            #   path =,
            #   rds =
            #   min_cells = 100
            # ),
            list_param_report = list(
              adcode = input$codex,
              analyse_type = input$analyse_type,
              annotation_column = input$selected_annotation,
              batch = input$selected_batch,
              cluster_id = "cluster_id",
              col_block = input$selected_block_id,
              cols_design = selected_values_design(),
              cols_design_GLMM_fixed = selected_values_design(),
              cols_design_GLMM_random = selected_values_GLMM_random(),
              conditions = input$selected_conditions,
              fc = input$choose_fc,
              # individuals = input$selected_individuals,
              log_bar = input$logex,
              min_cells = input$min_cells,
              min_samples = input$min_samples,
              path = r_global$path_to_data, # "/home/lohmann/analyticyte_first"
              p_value = input$given_pvalue,
              plot_legends = selected_values_string(),
              project = {
                if (!is.na(title)) title else ""
              },
              rds = "dataFE.rds",
              show_parameters = input$show_param_yaml,
              show_summary = input$show_summary_data,
              sorted = input$sortex,
              # subset_column= input$,
              transform = input$transform
            ),
            output_dir = paste(projects_folder(), r_global$project_folder_name, sep = "/"), # expect projects/p0001
            hash_tag = step_row$hash_tag, # expect an hash_tag
            format = "html",
            extension = "crcm",
            o_report = NULL
          )
        })
        showNotification("HTML created", type = "message")
        reloadTrigger(reloadTrigger() + 1)
        finish_step(project = r_global$project_id, SID = step_row$SID)

        # Save the info about the report
        r_global$report_dir <- res_info_report$output_dir_report
        r_global$report_file <- res_info_report$output_file_report
        r_global$yaml_file <- res_info_report$output_yaml
      }
    )
  })
}



## To be copied in the UI
# mod_qc_ui("qc_1")

## To be copied in the server
# mod_qc_server("qc_1")
