#' dataset UI Function
#'
#' @description A shiny Module.
#'
#' @param id,input,output,session Internal parameters for {shiny}.
#'
#' @noRd
#'
#' @importFrom shiny NS tagList
mod_dataset_ui <- function(id) {
  ns <- NS(id)
  tagList(
    h2("Dataset Information", class = "module-title-bold"),
    fluidRow(
      column(
        width = 6,
        div(DT::dataTableOutput(ns("fcs_info")) , class='table1')
      ),
      column(
        width = 4,
        DT::dataTableOutput(ns("clusters_info"))
      )
    ),
    fluidRow(
      column(
        width = 4,
        DT::dataTableOutput(ns("markers_info"))
      )
    )
  )
}

#' dataset Server Functions
#'
#' @noRd
mod_dataset_server <- function(id, r_global, reloadTrigger) {
  moduleServer(id, function(input, output, session) {
    ns <- session$ns
    dataFE <- reactiveVal()
    dataFE(r_global$data)


    fcs_data <- reactive({
      req(dataFE())
      dataFE()$SE_abundance@colData |> as.data.frame()
    })
    clusters_data <- reactive({
      req(dataFE())
      dataFE()$SE_abundance@elementMetadata |> as.data.frame()
    })
    markers_data <- reactive({
      req(dataFE())
      req("annot_markers" %in% names(dataFE()$SE_mfi@metadata))

      dataFE()$SE_mfi@metadata$annot_markers |> as.data.frame()
    })

    output$fcs_info <- DT::renderDataTable(
      {
        prettyDT(fcs_data(),
          selection = "multiple",
          caption = "FCS information table"
        )
      },
      server = F
    )

    output$clusters_info <- DT::renderDataTable(
      {
        prettyDT(clusters_data(),
          rownames = FALSE, selection = "multiple",
          caption = "Cluster information table"
        )
      },
      server = F
    )

    output$markers_info <- DT::renderDataTable(
      {
        prettyDT(markers_data(),
          rownames = FALSE, selection = "multiple",
          caption = "Marker information table"
        )
      },
      server = F
    )
    reloadTrigger()
  })
}

## To be copied in the UI
# mod_dataset_ui("dataset_1")

## To be copied in the server
# mod_dataset_server("dataset_1")
