#' export_ph UI Function
#'
#' @description A shiny Module.
#'
#' @param id,input,output,session Internal parameters for {shiny}.
#'
#' @noRd
#'
#' @importFrom shiny NS tagList
mod_export_ph_ui <- function(id, r_global) {
  ns <- NS(id)
  tagList(
    div(
      id = id,
      fluidRow(
        box(
          width = 12, status = "primary",
          div(
            column(
              width = 6, strong("Project of Reference: "),
              span(style = "color: #007bff;", r_global$project_folder_name) # Styling for the text
            ),
            column(
              width = 6, strong("Folder of Reference: "),
              span(style = "color: #007bff;", r_global$dataFE) # Styling for the text
            )
          )
        )
      ),
      fluidRow(
        column(
          width = 12,
          h2("Phantasus table export", class = "module-title-bold")
        )
      ),
      box(
        width = 7, title = "Phantasus tables", solidHeader = T, status = "primary",
        infoBox(
          title = "MFI split", width = 4,
          downloadButton(ns("downloadMFIsplit"), "Download")
        ),
        infoBox(
          title = "median of MFI", width = 4,
          downloadButton(ns("downloadMFImedian"), "Download")
        ),
        infoBox(
          title = "Abundance matrix", width = 4,
          downloadButton(ns("downloadAbundance"), "Download")
        )
      ),
      box(
        width = 5,
        status = "warning", solidHeader = T,
        title = "Export to Phantasus heatmap creator",
        markdown("
**Functionality Overview:**
This panel facilitates the export of your data to Excel format for advanced analysis in Phantasus.

**Downloadable Excel Files:**

   - **MFI Split by Markers:** Export the MFI data, providing detailed insights split by each marker.
   - **Median of MFI:** Obtain the median MFI values for each FCS file and marker.
   - **Abundance Matrix:** Generate an abundance matrix displaying the percentage of cells in each cluster for each FCS file.

")
      )
    )
  )
}

#' export_ph Server Functions
#'
#' importFrom analycyte.utils getFeature
#' importFrom openxlsx write.xlsx
#' @noRd
mod_export_ph_server <- function(id, r_global, reloadTrigger) {
  moduleServer(id, function(input, output, session) {
    ns <- session$ns
    dataFE <- reactiveVal()
    dataFE(r_global$data)

    # Create raw MFI list
    MFI_split_raw <- reactive({
      req(dataFE())
      scaled_mfi_all <- analycyte.utils::getFeature(dataFE()[["SE_mfi"]], c("assays")) |> as.list()

      return(scaled_mfi_all)
    })

    MFI_median_raw <- reactive({
      req(dataFE())
      mean_mfi <- analycyte.utils::getFeature(dataFE()[["SE_mfi"]], c("metadata", "scaled_MFI"))

      return(mean_mfi)
    })

    AbundanceCells <- reactive({
      req(dataFE())

      SE_temp <- dataFE()[["SE_abundance"]]
      counts <- analycyte.utils::getFeature(SE_temp, c("assays", "perCellCounts")) |> as.data.frame()
      return(counts)
    })


    output$downloadMFIsplit <- downloadHandler(
      filename = function() {
        paste(r_global$dataFE, "_MFI_split", ".xlsx", sep = "")
      },
      content = function(file) {
        openxlsx::write.xlsx(MFI_split_raw(), file = file, rowNames = T)
      }
    )

    output$downloadMFImedian <- downloadHandler(
      filename = function() {
        paste(r_global$dataFE, "_median_MFI", ".xlsx", sep = "")
      },
      content = function(file) {
        openxlsx::write.xlsx(MFI_median_raw(), file = file, rowNames = F)
      }
    )

    output$downloadAbundance <- downloadHandler(
      filename = function() {
        paste(r_global$dataFE, "_abundance", ".xlsx", sep = "")
      },
      content = function(file) {
        openxlsx::write.xlsx(AbundanceCells(), file = file, rowNames = T)
      }
    )
  })
}

## To be copied in the UI
# mod_export_ph_ui("export_ph_1")

## To be copied in the server
# mod_export_ph_server("export_ph_1")
