#' extract_sce UI Function
#'
#' @description A shiny Module.
#'
#' @param id,input,output,session Internal parameters for {shiny}.
#'
#' @noRd
#'
#' @importFrom shiny NS tagList
mod_extract_sce_ui <- function(id, r_global) {
  ns <- NS(id)
  tagList(
    fluidRow(
      box(
        width = 12, status = "primary",
        div(
          column(
            width = 6, strong("Project of Reference: "),
            span(style = "color: #007bff;", r_global$project_folder_name) # Styling for the text
          ),
          column(
            width = 6, strong("Cell Set of Reference: "),
            span(style = "color: #007bff;", r_global$dataSCE) # Styling for the text
          )
        )
      )
    ),
    fluidRow(
      column(
        width = 5,
        h2("FCS file import", class = "module-title-bold")
      ),
      column(width = 4),
      column(
        width = 3, br(),
       uiOutput(ns("extract_Features_button")),
       br()
      )
    ),
    fluidRow(box(
      status = "warning",
      title = "Extract", width = 12,
      markdown('
**Functionality Overview:**
This panel allows users to extract features from a specified cell set. Users can define State Markers and choose their preferred clustering method.

**Instructions:**
1. **Specify State Markers:**
   - Use the provided input field to specify State Markers.
2. **Select Clustering Method:**
   - Choose a clustering method from the dropdown menu. This selection determines how the cells will be grouped.
3. **Run Feature Extraction:**
   - Once you have specified the State Markers and selected a clustering method, click on the "Extract Features" button to initiate the process.

**Additional Information:**
- Accurate specification of State Markers and the appropriate choice of clustering method are essential for effective feature extraction.
')
      
    )),
    fluidRow(
      box(
        width = 12,
        column(
          width = 6,
          uiOutput(ns("Focused_Markers"))
        ),
        column(
          width = 1
        ),
        column(
          width = 4,
          uiOutput(ns("Colnames_markers")),
          uiOutput(ns("Focused_Cluster")),
          uiOutput(ns("Repeated_Id"))
        ),
        column(
          width = 1
        )
      )
    ),
    fluidRow(
      box(
        width = 12, title = "extract sce Parameters",
        div(
          class = "magnificationDiv", style = "position:absolute; right: 0px; top: 0px;",
          actionBttn(
            inputId = "extractFcsDisplay", style = "minimal",
            color = "danger", class = "magnifButton", label = icon("minus"), value = FALSE
          )
        ),
        conditionalPanel(
          condition = "input.extractFcsDisplay %2 == 1",
          verbatimTextOutput(outputId = ns("params_fcs_extract_object_see"))
        )
      )
    ),
    fluidRow(column(12, verbatimTextOutput(ns("state_extracted"))))
  )
}

#' extract_sce Server Functions
#'
#' @importFrom shinyWidgets checkboxGroupButtons
#' @importFrom  analycyte.utils getFeature
#' @noRd
mod_extract_sce_server <- function(id, r_global, reloadTrigger) {
  moduleServer(id, function(input, output, session) {
    ns <- session$ns
    dataSCE <- reactiveVal()
    dataExtracted <- reactiveVal()
    ExtractedSCE <- reactiveVal()

    # print("heho")
    # print(r_global$data_sce)
    dataSCE(r_global$data_sce)

    output$Focused_Markers <- renderUI({
      req(list_Markers())
      req(input$selected_name_marker)
      vector_of_interest <- list_Markers()[[input$selected_name_marker]]|>unname()
      checkboxGroupButtons(
        inputId = ns("selected_marker_focused"),
        label = "Markers to extract:",
        choices = vector_of_interest,
        selected = vector_of_interest,
        checkIcon = list(
          yes = tags$i(
            class = "fa fa-check-square"
            # ,style = "color: steelblue"
          ),
          no = tags$i(
            class = "fa fa-square-o"
           # , style = "color: steelblue"
          )
        )
      )
    })

    list_Markers <- reactive({
      req(dataSCE())
      # converted_markers_table <- SummarizedExperiment::rowData(dataSCE()) |> as.data.frame()
      converted_markers_table <- getFeature(dataSCE(), c("rowData")) |> as.data.frame()
      return(converted_markers_table)
      
    })
    
    
    output$Colnames_markers <-renderUI({
      req(list_Markers())

      pickerInput(
        inputId = ns("selected_name_marker"),
        label = "Marker ID to extract:",
        choices = list_Markers() |> colnames(),
        selected = "desc",
        multiple = TRUE,
        options = pickerOptions(maxOptions = 1)
      )
    })
    
    output$Focused_Cluster <- renderUI({
      req(dataSCE())

      pickerInput(
        inputId = ns("selected_cluster_focused"),
        label = "Cluster to extract:",
        choices = analycyte.utils::getFeature(dataSCE(), c("colData")) |> colnames(),
        selected = "Phenograph_F7C8"
      )
    })
    output$Repeated_Id <- renderUI({
      req(dataSCE())

      pickerInput(
        inputId = ns("selected_id_for_sce"),
        label = "ID Fcs:",
        choices = analycyte.utils::getFeature(dataSCE(), c("colData")) |> colnames(),
        selected = "fcsID"
      )
    })

    output$extract_Features_button <- renderUI({
      req(dataSCE())

      tagList(
        fluidRow(
          column(12,
            align = "center",
            # actionBttn(inputId = ns("extractfromSCE"), label = "Extract Features")
            
            actionBttn(
              inputId = ns("extractfromSCE"),
              class = "btn-secondary",
              icon = icon("gears"),
              style = "material-flat",
              label = "Extract Features(5: fe)"
            )
            
          )
        )
      )
    })


    param_extract_fcs <- reactive({
      req(dataSCE())
      obj <- list(

        # return a list with abundance and or mfis
        "sce" = dataSCE(),
        "selected" = "counts", # counts is not transformed
        "extract_mfi" = TRUE,
        "convert_row_name" =  input$selected_name_marker ,
        "extract_abund" = TRUE,
        "list_formats" = list("abundance" = "wide", "mfi" = "long"), # , "colors" = NULL # wide or long
        "fcs_ids" = "patient",
        "clustering" = input$selected_cluster_focused, # "Phenograph_F7C8"
        "id" = input$selected_id_for_sce, # "fcsID"
        # "vector_markers" = input$selected_marker_focused # all_col
        "vector_markers"=  list_Markers()[["fcs_colname"]]
      )

      return(obj)
    })
    output$params_fcs_extract_object_see <- renderPrint({
      req(param_extract_fcs())
      print(param_extract_fcs())
    })


    observeEvent(input$extractfromSCE, {
      # browser()

      if (is.null(param_extract_fcs()[["vector_markers"]])) {
        showNotification("please select some markers", type = "error")
        return()
      }

      sub_sce <- param_extract_fcs()[["sce"]][param_extract_fcs()[["vector_markers"]], ]
      # sub_sce <- param_extract_fcs()[["sce"]]
      # if (!fcs_extracted()) {
      # browser()
      extraction <- analycyte.utils::extract_features_sce(
        sce = sub_sce[, which(!colSums(sub_sce@assays@data[["counts"]]) <= 0)],
        selected = param_extract_fcs()[["selected"]],
        convert_row_name = param_extract_fcs()[["convert_row_name"]],
        extract_mfi = param_extract_fcs()[["extract_mfi"]],
        extract_abund = param_extract_fcs()[["extract_abund"]],
        list_formats = param_extract_fcs()[["list_formats"]],
        fcs_ids = param_extract_fcs()[["fcs_ids"]],
        clustering = param_extract_fcs()[["clustering"]],
        id = param_extract_fcs()[["id"]]
      )

      dataExtracted(extraction)
      # fcs_extracted(TRUE)
      showNotification("features extracted", type = "message")
      # } else {
      #   showNotification("features already extracted", type = "message")
      # }

      # print(extraction)
      print(dataExtracted())


      list_formats_ <- list(
        abundance = extraction$abund_tbl$params$format,
        mfi = extraction$mfis_tbl$params$format,
        metadata = "wide",
        markers = NULL
      )
      grouping_colname <- list(
        abundance = extraction$abund_tbl$params$id,
        mfi = extraction$mfis_tbl$params$id,
        metadata = param_extract_fcs()[["id"]],
        # metadata = "identification",
        markers = NULL
      )
      fcs_ids <- "patient"
      parsing <- list("sep" = "#", "pos_ids" = c(1))

      target_long_columns <- list(
        "marker" = param_extract_fcs()[["convert_row_name"]],
        "cluster" = param_extract_fcs()[["clustering"]],
        # "value" ="identification",
        "value" = param_extract_fcs()[["id"]],
        "mfi" = "median_value"
      )

      abundance <- extraction$abund_tbl$table
      meta <- dataSCE()@metadata$experiment_info

      # recalc n cells
      meta$n_cells <- abundance[-1] |> rowSums()
      # dataSCE()@metadata$experiment_info

      # print(meta)

      mfi <- extraction$mfis_tbl$table

      is_scaled <- TRUE

      to_scale <- FALSE

      color <- NULL

      markers <- NULL

      applied_fct <- "asinh"

      asinh_Ab <- 0.03

      asinh_MFI <- 5

      state_marker <- NULL



      #
            analycyte.utils::create_se(
              list_formats = list_formats_, # , "colors" = NULL # wide or long
              abundance = abundance,
              mfi = mfi,
              is_scaled = is_scaled,
              to_scale = to_scale,
              meta = meta,
              color = color,
              markers = markers,
              annot_markers = getFeature(dataSCE(), c("rowData")),
              target_long_columns = target_long_columns, #
              fcs_ids = fcs_ids,
              parsing = parsing, # list(sep="_",pos_ids=c(1,2)), # position fcs id
              grouping_colname = grouping_colname, # , "colors" = NULL
              applied_fct = applied_fct,
              column_with_ncells = "n_cells",
              asinh_Ab = asinh_Ab, # à reflechir , dependance asinh? que faire si c'est une autre transformation?
              asinh_MFI = asinh_MFI,
              state_marker = state_marker
            ) -> r_global$data
      #
            ExtractedSCE(r_global$data)
      #
      #       print(meta)
      #
            showNotification("FE extracted", type = "message")
            reloadTrigger(reloadTrigger() + 1)
            
            new_step(project = r_global$project_id, label = "mod_extract_sce", output_type = "fe" )
            
            last_SID <- steps() |>
              dplyr::pull(.data$SID) |>
              max()
            
            step_row <- steps() |> dplyr::filter(.data$SID == last_SID)
            project_row <- projects(verbose = T) |> dplyr::filter(.data$id == step_row$PID)
            
            # recuperation du hash_tag
            
            hash <- step_row |> dplyr::pull(.data$hash_tag)
            project_path <- project_row |> dplyr::pull(.data$path)
            
            path <- unclass(fs::path(project_path, hash))
            
            # ecriture du data sur le disque
            
            if (dir.exists(path)) {
              saveRDS(r_global$data, file = file.path(path, "dataFE.rds"))
              showNotification("RDS saved", type = "message")
              reloadTrigger(reloadTrigger() + 1)
            } else {
              showNotification("No project hash folder found.", type = "warning")
              reloadTrigger(reloadTrigger() + 1)
            }
            
            finish_step(project = r_global$project_id, SID = last_SID)
            
      # browser()
    })
  })
}

## To be copied in the UI
# mod_extract_sce_ui("extract_sce_1")

## To be copied in the server
# mod_extract_sce_server("extract_sce_1")
