#' import_feature UI Function
#'
#' @description A shiny Module.
#'
#' @param id,input,output,session Internal parameters for {shiny}.
#'
#' @noRd
#'
#' @import shiny
#' @importFrom shinyWidgets awesomeCheckbox actionBttn
#' @import shinydashboard
#' @importFrom shinycssloaders withSpinner
mod_import_feature_ui <- function(id, r_global) {
  ns <- NS(id)
  tagList(
    fluidRow(
      box( width = 12, status = "primary",
           div(
             column(
               width = 6, strong("Project of Reference: "), 
               span(style = "color: #007bff;", r_global$project_folder_name) # Styling for the text
             )
           )
      )
      
    ),
    fluidRow(
      column(
        width = 12,
        h2("Features file import", class = "module-title-bold")
      )
    ),
    fluidRow(box(
      status = "warning",solidHeader = T, collapsible = T, collapsed = F,
      title = "Import from Feature", width = 12,
      markdown('
**Functionality Overview:**
The "Import from Feature" panel allows users to import various data tables such as abundance, MFI (Mean Fluorescence Intensity), and metadata tables. Users also have the option to load a marker table. All these tables should be linked by a common ID.

**Instructions:**
1. **Browse and Select Files:**
   - Click on the \'Browse\' button to select the data files you wish to import.
   - You can import files for abundance data, MFI data, and metadata.
   - Optionally, you can also load a marker table. (not available yet)
2. **Ensure Common ID Column:**
   - Make sure that each table has a common ID column. This ID is essential for creating the bound between the different tables.
3. **Upload Files:**
   - After selecting the files, complete widgets.

**Additional Information:**
- This import functionality is designed especially for OMIQ exported features.
')
      
    )),
    br(),
    fluidRow(
      box(
        solidHeader = T,
        title = "Generalized Parameters", width = 3,


        # Button to toggle advanced settings
        actionBttn("toggleSettings", "Advanced Settings",
          style = "minimal", color = "primary", class = "advanced-settings-button"
        ),
        br(),
        conditionalPanel(
          # condition = "input.toggleSettings %2 == 1",
          condition = "input.toggleSettings %2 == 0",
          fluidRow(
            column(
              width = 3,
              div(class = "InputSized", style = "width: 150px", uiOutput(ns("fcs_ids_type"))),
              div(class = "InputSized", style = "width: 100px", uiOutput(ns("parsing_abund_sep")))
            ),
            column(width = 4, offset = 0, style = "padding:0px;"),
            column(
              width = 5,
              div(class = "InputSized", style = "width: 50px", uiOutput(ns("conditional_section")))
            )
          ),
          br(),
          fluidRow(
            column(
              width = 4,
              div(class = "InputSized", uiOutput(ns("remove_fcs_in_names")))
            ),
            column(
              width = 4,
              div(class = "InputSized", uiOutput(ns("omiq_colnames")))
            )
          )
        ),
        br(),
        fluidRow(
          column(
            width = 6,
            div(style = "width: 75px;", uiOutput(ns("applied_fct")))
          ),
          column(
            width = 6,
            div(style = "width: 100px;", uiOutput(ns("asinh_UI")))
          )
        ),
        br(),
        uiOutput(ns("createFE_button"))
      ),
      tabBox(
        width = 9, side = "right",
        title = "Features Loading",
        id = "feat_load",
        tabPanel("Load",
          id = "first_page",
          tagList(
            fluidRow(
              box(
                status = "primary",
                width = 6, title = span(icon("clipboard-list"), "Abundance table"),
                solidHeader = FALSE,
                datamods::import_file_ui(
                  title = "Abundance", ns("import-abundance"),
                  file_extensions = c(".csv", ".txt", ".xls", ".xlsx"),
                  preview_data = F
                ),
                withSpinner(DT::dataTableOutput(outputId = ns("data_abun")) , type=6 ),
                br(),
                fluidRow( ## fluidrow abundance ####
                  column(
                    width = 4,
                    tags$div(class = "InputSized", uiOutput(ns("sharedColAb_UI")))
                  ),
                  column(
                    width = 4,
                    tags$div(class = "InputSized", uiOutput(ns("fileformat_abund")))
                  ), column(
                    width = 4,
                    tags$div(class = "InputSized", uiOutput(ns("percent_UI")))
                  )
                )
              ),
              box(
                width = 6, status = "primary",
                title = span(icon("clipboard-list"), "MFIs table"),
                solidHeader = FALSE,
                datamods::import_file_ui(
                  title = "MFIs", ns("import-mfis"), preview_data = F,
                  file_extensions = c(".csv", ".txt", ".xls", ".xlsx")
                ),
                withSpinner(DT::dataTableOutput(outputId = ns("data_mfi")), type =6),
                br(),
                fluidRow( ## fluidrow mfis ####
                  column(
                    width = 4,
                    tags$div(class = "InputSized", uiOutput(ns("sharedColMFI_UI"))) # "id_col"
                  ),
                  column(
                    width = 4,
                    tags$div(class = "InputSized", uiOutput(ns("fileformat_mfi")))
                  ),
                  column(
                    width = 3,
                    tags$div(class = "InputSized", uiOutput(ns("is_scaled_mfi"))), # true or false

                    tags$div(class = "InputSized", uiOutput(ns("to_scale_mfi")))
                  )
                ),
                fluidRow(tags$div(uiOutput(ns("conditionnal_asinh_MFI")))), # 5000
                tags$div(uiOutput(ns("conditional_long_mfi"))) # if format mfi is long : need value mfi column name, cluter c name, marker c name
              )
            ),
            # ),
            #   ,
            fluidRow( ## metadata ####
              box(
                width = 6, status = "primary",
                title = span(icon("clipboard-list"), "Metadata table"),
                solidHeader = FALSE,
                datamods::import_file_ui(
                  title = "Metadata", ns("import-metadata"), preview_data = F,
                  file_extensions = c(".csv", ".txt", ".xls", ".xlsx")
                ),
                withSpinner(DT::dataTableOutput(outputId = ns("data_metadata")), type =6),
                br(),
                fluidRow( ## fluidrow mfis ####
                  column(
                    width = 4,
                    tags$div(class = "InputSized", uiOutput(ns("sharedColMeta_UI"))) # "id_col"
                  ),
                  column(
                    width = 4,
                    tags$div(class = "InputSized", uiOutput(ns("fileformat_metadata"))) # file_format = c("wide", "long"),
                  ),
                  column(
                    width = 4,
                    tags$div(class = "InputSized", uiOutput(ns("column_ncells")))
                  )
                )
              ),
              box(
                width = 6, status = "primary",
                title = span(icon("clipboard-list"), "Markers table"),
                solidHeader = FALSE,
                datamods::import_file_ui(
                  title = "Markers", ns("import-markers"), preview_data = F,
                  file_extensions = c(".csv", ".txt", ".xls", ".xlsx")
                ), br(),
                withSpinner(DT::dataTableOutput(outputId = ns("data_markers")), type=6)
              )
            )
          )
        )
      )
    ),
    fluidRow(
      box(
        width = 12, title = "create_se Parameters",
        div(
          class = "magnificationDiv", style = "position:absolute; right: 0px; top: 0px;",
          actionBttn(
            inputId = "paramsDisplay", style = "minimal",
            color = "danger", class = "magnifButton", label = icon("minus"), value = FALSE
          )
        ),
        conditionalPanel(
          condition = "input.paramsDisplay %2 == 1",
          verbatimTextOutput(outputId = ns("params_object_see"))
        )
      )
    )
  )
}

#' import_feature Server Functions
#'
#' @noRd
mod_import_feature_server <- function(id, r_global, reloadTrigger) {
  moduleServer(id, function(input, output, session) {
    ns <- session$ns

    state_load <- reactiveVal(FALSE)
    one_is_loaded <- reactiveVal(FALSE)
    feature_values_load <- reactiveVal(FALSE)
    data_downloaded <- reactiveVal(FALSE)
    dataFE <- reactiveVal()

    observe({
      how <- "success" %in% c(
        imported_abun$status(),
        imported_mfi$status(),
        imported_metadata$status()
      )
      one_is_loaded(how)
    })

    observe({
      how <- "success" %in% c(
        imported_abun$status(),
        imported_mfi$status()
      ) & "success" %in% c(imported_metadata$status())

      state_load(how)
    })




    observe({
      how <- "success" %in% c(
        imported_abun$status(),
        imported_mfi$status()
      )
      feature_values_load(how)
    })


    defaultValue <- reactiveValues(
      clustCol = "filter",
      markerCol = "ftr (sec)",
      mfiValueCol = "median_value",
      sharedCol = "file",
      totalCell = "Unfiltered|count",
      file_format_abund = "wide",
      file_format_mfi = "long",
      fcs_names = "patient",
      ppatient = 1,
      pcluster = 1,
      is_scaled = T,
      to_scaled = F,
      fcs_to_cut = F
    )


    imported_abun <- datamods::import_file_server(
      id = "import-abundance",
      return_class = "data.frame",
      trigger_return = "change"
    )

    imported_mfi <- datamods::import_file_server(
      id = "import-mfis",
      return_class = "data.frame",
      trigger_return = "change"
    )

    imported_metadata <- datamods::import_file_server(
      id = "import-metadata",
      return_class = "data.frame",
      trigger_return = "change"
    )

    imported_markers <- datamods::import_file_server(
      id = "import-markers",
      return_class = "data.frame",
      trigger_return = "change"
    )

    output$percent_UI <- renderUI({
      req(imported_abun$data())
      shinyWidgets::awesomeCheckbox(ns("check_rawPercent"), label = HTML("<b>Percentage<b>"), value = FALSE)
    })

    output$sharedColAb_UI <- renderUI({
      req(imported_abun$data())
      pickerInput(inputId = ns("sharedColAbundance"), "Shared column:", choices = colnames(imported_abun$data()), selected = defaultValue$sharedCol)
    })
    # file_format = c("wide", "long"), # ex: wide chez OMIQ, long chez FJ
    output$fileformat_abund <- renderUI({
      req(imported_abun$data())
      pickerInput(
        inputId = ns("format_abund"), "Table Format:", choices = c("wide", "long"),
        choicesOpt = list(
          icon = c("glyphicon-arrow-right", "glyphicon-arrow-down")
        ), selected = defaultValue$file_format_abund
      )
    })


    output$data_abun <- DT::renderDataTable(
      imported_abun$data(),
      escape = FALSE,
      selection = "none", server = FALSE,
      options = list(dom = "t", scrollX = TRUE, ordering = FALSE)
    )

    output$data_mfi <- DT::renderDataTable(
      imported_mfi$data(),
      escape = FALSE, 
      selection = "none", server = FALSE,
      options = list(dom = "t", scrollX = TRUE, ordering = FALSE)
    )

    output$data_metadata <- DT::renderDataTable(
      imported_metadata$data(),
      escape = FALSE, 
      selection = "none", server = FALSE,
      options = list(dom = "t", scrollX = TRUE, ordering = FALSE)
    )

    output$data_markers <- DT::renderDataTable(
      imported_markers$data(),
      escape = FALSE,
      selection = "none", server = FALSE,
      options = list(dom = "t", scrollX = TRUE, ordering = FALSE)
    )

    datamods::import_file_server("import-mfis", trigger_return = "change")

    output$sharedColMFI_UI <- renderUI({
      req(imported_mfi$data())

      selectInput(inputId = ns("sharedColMFI"), "Shared column:", choices = colnames(imported_mfi$data()), selected = defaultValue$sharedCol)
    })
    #
    output$fileformat_mfi <- renderUI({
      req(imported_mfi$data())
      pickerInput(
        inputId = ns("format_mfi"), "Table Format:", choices = c("wide", "long"),
        choicesOpt = list(
          icon = c("glyphicon-arrow-right", "glyphicon-arrow-down")
        ),
        selected = defaultValue$file_format_mfi
      )
    })


    output$to_scale_mfi <- renderUI({
      req(imported_mfi$data())
      shinyWidgets::awesomeCheckbox(ns("check_is_need_scale"), label = HTML("<b>need to scale<b>"), value = defaultValue$to_scaled)
    })

    output$is_scaled_mfi <- renderUI({
      req(imported_mfi$data())
      shinyWidgets::awesomeCheckbox(ns("check_is_scaled_mfi"), label = HTML("<b>is scaled<b>"), value = defaultValue$is_scaled)
    })

    output$sharedColMeta_UI <- renderUI({
      req(imported_metadata$data())
      selectInput(inputId = ns("sharedColMeta"), "Shared column:", choices = colnames(imported_metadata$data()), selected = defaultValue$sharedCol)
    })

    output$fileformat_metadata <- renderUI({
      req(imported_metadata$data())
      pickerInput(
        inputId = ns("format_metadata"), "Table Format:", choices = c("wide"),
        choicesOpt = list(
          icon = c("glyphicon-arrow-right")
        ),
        selected = "wide"
      )
    })

    output$column_ncells <- renderUI({
      req(imported_metadata$data())
      pickerInput(
        inputId = ns("num_cells_col"), "Number of cells column:", choices = c(colnames(imported_metadata$data())),
        selected = defaultValue$totalCell
      )
    })

    output$fcs_ids_type <- renderUI({
      req(one_is_loaded())
      pickerInput(ns("fcs_names"),
        # pickerInput("fcs_names",
        label = "contained in fcs:",
        choices = list("patient", "cluster", "patient_cluster"),
        selected = defaultValue$fcs_names
      )
    })

    output$parsing_abund_sep <- renderUI({
      req(one_is_loaded())

      selectInput(ns("separator_name"),
        label = "character sep:",
        choices = list("_", "-", " ", "#"),
        selected = "#"
      )
    })

    output$conditional_section <- renderUI({
      req(one_is_loaded())

      selected_option <- input$fcs_names

      tagList(
        fluidRow(
          column(width = 4.5, conditionalPanel(
            condition = 'input.fcs_names != "patient"',
            numericInput(ns("position_cluster"), label = "Cluster field:", value = defaultValue$pcluster, min = 1),
            ns = ns
          )),
          column(width = 4.5, conditionalPanel(
            condition = 'input.fcs_names != "cluster"',
            numericInput(ns("position_patient"), label = "Patient field:", value = defaultValue$ppatient, min = 1),
            ns = ns
          ))
        )
      )
    })
    # applied_fct = "asinh",
    output$applied_fct <- renderUI({
      req(feature_values_load())
      pickerInput(
        inputId = ns("transformation_fct"), "transformation:",
        choices = c("asinh"), selected = "asinh"
      )
    })

    # fcs_split = NULL,
    output$remove_fcs_in_names <- renderUI({
      req(one_is_loaded())
      awesomeCheckbox(ns("check_remove_fcs_labs"), label = HTML("<b>fcs cut<b>"), value = defaultValue$fcs_to_cut)
    })

    # omiq_column_name=F
    output$omiq_colnames <- renderUI({
      req(imported_abun$data())
      awesomeCheckbox(ns("check_is_colname_omiq"), label = HTML("<b>omiq colname type<b>"), value = TRUE)
    })


    ### MFIS ####


    output$conditionnal_asinh_MFI <- renderUI({
      req(imported_mfi$data())

      # numericInput(inputId = 'num_asinh_MFI', 'Chose asinh value', value = 5000, step = 1)

      tagList(
        # fluidRow(
        column(width = 4, conditionalPanel(
          condition = "input.check_is_need_scale",
          numericInput(inputId = ns("num_asinh_MFI"), "Chose asinh value", value = 5000, step = 1),
          ns = ns
        ))
        # )
      )
    })

    output$asinh_UI <- renderUI({
      req(feature_values_load())

      numericInput(inputId = ns("Ab_aSinh"), "Chose asinh value", value = 0.03, step = 0.01)
    })


    output$conditional_long_mfi <- renderUI({
      req(imported_mfi$data())

      tagList(
        fluidRow(
          column(width = 4, conditionalPanel(
            condition = 'input.format_mfi == "long"',
            pickerInput(
              ns("mfi_col_sample"),
              label = "fcs:",
              choices = colnames(imported_mfi$data()),
              selected = defaultValue$sharedCol
            ),
            ns = ns
          )),
          column(width = 4, conditionalPanel(
            condition = 'input.format_mfi == "long"',
            pickerInput(
              ns("mfi_col_cluster"),
              label = "Cluster:",
              choices = colnames(imported_mfi$data()),
              selected = defaultValue$clustCol
            ),
            ns = ns
          )),
          column(width = 4, conditionalPanel(
            condition = 'input.format_mfi == "long"',
            pickerInput(
              ns("mfi_col_marker"),
              label = "Marker:",
              choices = colnames(imported_mfi$data()),
              selected = defaultValue$markerCol
            ),
            ns = ns
          )),
          column(width = 4, conditionalPanel(
            condition = 'input.format_mfi == "long"',
            pickerInput(
              ns("mfi_col_mfi"),
              label = "Value:",
              choices = colnames(imported_mfi$data()),
              selected = defaultValue$mfiValueCol
            ),
            ns = ns
          ))
        )
      )
    })

    datamods::import_file_server("import-metadata", trigger_return = "change")

    ### MARKERS ####

    datamods::import_file_server("import-markers", trigger_return = "change")



    param_list <- reactive({
      obj <- list(
        "list_formats" = list(
          "abundance" = input$format_abund,
          "mfi" = input$format_mfi,
          "metadata" = input$format_metadata,
          "markers" = NULL
        ),
        "grouping_colname" = list(
          "abundance" = input$sharedColAbundance,
          "mfi" = input$sharedColMFI,
          "metadata" = input$sharedColMeta,
          "markers" = NULL
        ),
        "parsing" = list(
          sep = input$separator_name,
          pos_ids = {
            if (is.null(input$fcs_names)) {
              NULL
            } else if (input$fcs_names == "patient") {
              c(input$position_patient)
            } else if (input$fcs_names == "cluster") {
              c(input$position_cluster)
            } else {
              c(input$position_patient, input$position_cluster)
            }
          }
        ),
        "target_long_columns" = list(
          "value" = input$mfi_col_sample,
          "cluster" = input$mfi_col_cluster,
          "marker" = input$mfi_col_marker,
          "mfi" = input$mfi_col_mfi
        ),
        "fcs_ids" = input$fcs_names,
        "abundance" = imported_abun$data(),
        "mfi" = imported_mfi$data(),
        "is_scaled" = input$check_is_scaled_mfi,
        "to_scale" = input$check_is_need_scale,
        "meta" = imported_metadata$data(),
        "color" = NULL,
        "markers" = NULL,
        "applied_fct" = input$transformation_fct,
        "asinh_Ab" = input$Ab_aSinh,
        "fcs_split" = input$check_remove_fcs_labs,
        "is_percent" = input$check_rawPercent,
        "asinh_MFI" = input$num_asinh_MFI,
        "state_marker" = NULL,
        "omiq_column_name" = input$check_is_colname_omiq,
        "column_with_ncells" = input$num_cells_col
      )

      return(obj)
    })


    output$params_object_see <- renderPrint({
      req(param_list())
      print(param_list())
    })

    # launch SE with features tables

    output$createFE_button <- renderUI({
      req(state_load())

      tagList(
        fluidRow(
          column(12,
            align = "center",
            actionBttn(inputId = ns("runFE"), label = "Create the Analysis object")
          )
        )
      )
    })

    observeEvent(input$runFE, {
      r_global$data <- analycyte.utils::create_se(
        list_formats = param_list()[["list_formats"]],
        abundance = param_list()[["abundance"]],
        mfi = param_list()[["mfi"]],
        is_scaled = param_list()[["is_scaled"]],
        to_scale = param_list()[["to_scale"]],
        meta = param_list()[["meta"]],
        color = param_list()[["color"]],
        markers = param_list()[["markers"]],
        target_long_columns = param_list()[["target_long_columns"]],
        fcs_ids = param_list()[["fcs_ids"]],
        parsing = param_list()[["parsing"]],
        grouping_colname = param_list()[["grouping_colname"]],
        applied_fct = param_list()[["applied_fct"]],
        asinh_Ab = param_list()[["asinh_Ab"]],
        column_with_ncells = param_list()[["column_with_ncells"]],
        fcs_split = param_list()[["fcs_split"]],
        is_percent = param_list()[["is_percent"]],
        asinh_MFI = param_list()[["asinh_MFI"]],
        state_marker = param_list()[["state_marker"]],
        omiq_column_name = param_list()[["omiq_column_name"]]
      )

      showNotification("FE loaded", type = "message")
      reloadTrigger(reloadTrigger() + 1)

      new_step(project = r_global$project_id, label = "mod_import_feature", output_type = "fe")

      last_SID <- steps() |>
        dplyr::pull(.data$SID) |>
        max()

      step_row <- steps() |> dplyr::filter(.data$SID == last_SID)
      project_row <- projects(verbose = T) |> dplyr::filter(.data$id == step_row$PID)

      # recuperation du hash_tag

      hash <- step_row |> dplyr::pull(.data$hash_tag)
      project_path <- project_row |> dplyr::pull(.data$path)

      path <- unclass(fs::path(project_path, hash))

      # ecriture du data sur le disque

      if (dir.exists(path)) {
        saveRDS(r_global$data, file = file.path(path, "dataFE.rds"))
        showNotification("RDS saved", type = "message")
        reloadTrigger(reloadTrigger() + 1)
      } else {
        showNotification("No project hash folder found.", type = "warning")
        reloadTrigger(reloadTrigger() + 1)
      }

      finish_step(project = r_global$project_id, SID = last_SID)



      #   # data$path <- as.character(parseDirPath(volumes, input$directory))
      #   # data$project <- input$name_project
      #   # data$rds <- input$name_rds_to_save
      #   dataFE(data)
      #   data_downloaded(TRUE)
      #   showNotification("FE loaded", type = "message")
      # } else {
      #   showNotification("FE already loaded", type = "message")
      # }
    })
  })
}

## To be copied in the UI
# mod_import_feature_ui("import_feature_1")

## To be copied in the server
# mod_import_feature_server("import_feature_1")
