#' qc UI Function
#'
#' @description A shiny Module.
#'
#' @param id,input,output,session Internal parameters for {shiny}.
#'
#' @noRd
#' @import bsplus
#' @importFrom shiny NS tagList
#' @importFrom shinyWidgets pickerOptions actionBttn
#' @importFrom shinycssloaders withSpinner
mod_qc_ui <- function(id, r_global) {
  ns <- NS(id)
  tagList(
    div(
      id = id,
      fluidRow(
        box(
          width = 12, status = "primary",
          div(
            column(
              width = 6, strong("Project of Reference: "),
              span(style = "color: #007bff;", r_global$project_folder_name) # Styling for the text
            ),
            column(
              width = 6, strong("Folder of Reference: "),
              span(style = "color: #007bff;", r_global$dataFE) # Styling for the text
            )
          )
        )
      ),
      fluidRow(
        column(
          width = 5,
          h2("Control Quality", class = "module-title-bold")
        ),
        column(width = 4),
        column(
          width = 3,
          class = "run_report",
          actionBttn(
            inputId = ns("run_qc"),
            class = "btn-secondary",
            icon = icon("gears"),
            style = "material-flat",
            label = "Run the report"
          )
        )
      ),
      fluidRow(
        column(
          width = 6,
          column(
            width = 6,
            radioButtons(ns("analyse_type"), "Analyse Type:", choices = c(
              "Quality Control" = "QC",
              "Control" = "CT"
            ), selected = "QC", inline = T)
          )
        )



        # box(width = 6,
        #     "path"
        # ),
        #
        # box(width = 6,
        #     "project"
        # ),
        # box(width = 6,
        #     "rds"
        # ),
      ),
      fluidRow(
        tabBox(
          height = 300,
          width = 7, side = "left",
          title = "",
          selected = "General Parameters",
          id = "param_set",
          tabPanel("TRUE/FALSE",
            id = "yes_no_set",
            tagList(
              fluidRow(
                column(
                  width = 12,
                  checkboxInput(ns("codex"), "Add code Info", value = FALSE),
                  checkboxInput(ns("logex"), "Log on graphs", value = FALSE),
                  checkboxInput(ns("sortex"), "Sort on graphs", value = FALSE),
                  checkboxInput(ns("show_param_yaml"), "Show the parameter table", value = TRUE),
                  checkboxInput(ns("show_summary_data"), "Show the summary tables", value = TRUE)
                )
              )
            )
          ),
          tabPanel("Tresholds",
            id = "thresh_set",
            tagList(
              fluidRow(
                column(
                  width = 6,
                  uiOutput(ns("o_min_cells"))
                ),
                # column(
                #   width = 6,
                #   uiOutput(ns("o_min_samples"))
                # # ),
                # column(
                #   width = 6,
                #
                #   sliderInput(ns("choose_fc"), "Fold Change threshold:",
                #               min = 0,
                #               max = 10,
                #               value = 1.0,
                #               step = 0.1
                #   )
                #
                #   # uiOutput(ns("fixed_fc"))
                # ),
                # column(
                #   width = 6,
                #   div(class = "InputSized", numericInput(ns("given_pvalue"), "pvalue cutoff:", 0.000001, min = 0, max = 1, step = 0.001))
                # )
              )
            )
          ),
          tabPanel("General Parameters",
            id = "select_target_set",
            tagList(
              fluidRow(
                column(
                  width = 6,
                  div(class = "InputSized", uiOutput(ns("conditions_")))
                ),
                column(
                  width = 6,
                  div(class = "InputSized", uiOutput(ns("batch_")))
                ),
                column(
                  width = 6,
                  uiOutput(ns("annot_on_plots_"))
                ),
                column(
                  width = 6, div(class = "InputSized", uiOutput(ns("annotation_metacluster")))
                )
                # ,box(
                #   width = 6,
                #   "subset_column"
                # )
              )
            )
          )
          # , tabPanel("Univariate",
          #    id = "univariate_set",
          #    tagList(
          #      fluidRow(
          #        box(
          #          width = 6,
          #          "col_block"
          #        ),
          #        box(
          #          width = 6,
          #          "cols_design"
          #        ),
          #        box(
          #          width = 6,
          #          "cols_design_GLMM_fixed"
          #        ),
          #        box(
          #          width = 6,
          #          "cols_design_GLMM_random"
          #        ), box(
          #          width = 6,
          #          "transform",
          #          checkboxInput(ns("transform"), "Transform data", value = FALSE)
          #        )
          #      )
          #    )
          #  )
        ),
        box(
          width = 5,
          status = "warning", solidHeader = T, collapsible = T, collapsed = F,
          title = "Quality Control module",
          markdown('

**Functionality Overview:**
The Quality Control Module provides an interactive interface for users to generate
customized quality control reports. Users can set parameters, define thresholds, and choose various options for their report.

**Instructions:**
1. **General Parameters:**
   - In the \'General Parameters\' tabset, you can use the widgets to choose general annotations for your report.
2. **Threshold Settings:**
   - Navigate to the \'Threshold\' tabset to set cutoffs for your analysis, like the minimum number of cells for each cluster.
3. **True/False Options:**
   - In the \'TRUE/FALSE\' tabset, use boolean widgets to customize your report.
   - Options may include showing the code used for analysis or ordering the graphs in a specific manner.
4. **Run the Report:**
   - After setting all parameters and options, click on the "RUN THE REPORT" button located at the top of the page.
   - This will initiate the generation of your report.
   - The report will take some time to generate so be patient, it will open in a new window.

**Additional Information:**
- Please ensure that your browser allows pop-up windows to be opened.

                   ')
        )
      ),
      fluidRow(
        column(
          width = 8,
          withSpinner( DT::dataTableOutput(ns("metadata_FE_running")) , type = 6)
        ),
        column(
          width = 4,
          withSpinner(  DT::dataTableOutput(ns("rowData_FE_running")), type = 6)
        )
      )
    )
  )
}

#' qc Server Functions
#' @import bsplus
#' @importFrom shiny NS tagList
#' @importFrom shinyWidgets pickerOptions
#' @noRd
mod_qc_server <- function(id, r_global, reloadTrigger) {
  moduleServer(id, function(input, output, session) {
    ns <- session$ns

    dataFE <- reactiveVal()
    # curent hash r_global$dataFE
    dataFE(r_global$data)


    meta_names <- reactiveVal()



    fcs_data <- reactive({
      req(dataFE())
      getFeature(dataFE()$SE_abundance, c("colData")) |> as.data.frame()
    })
    row_data <- reactive({
      req(dataFE())
      getFeature(dataFE()$SE_abundance, c("rowData")) |> as.data.frame()
    })

    output$metadata_FE_running <- DT::renderDataTable(
      {
        prettyDT(fcs_data(), selection = "multiple")
      },
      server = F
    )

    output$rowData_FE_running <- DT::renderDataTable(
      {
        prettyDT(row_data(), selection = "multiple", rownames = FALSE)
      },
      server = F
    )




    observe({
      req(dataFE())
      names <- analycyte.utils::getFeature(dataFE()$SE_abundance, c("colData"))
      meta_names(names)
    })
    col_names <- reactiveVal()

    observe({
      req(dataFE())
      names <- analycyte.utils::getFeature(dataFE()$SE_abundance, c("rowData"))
      col_names(names)
    })

    output$o_min_cells <- renderUI({
      req(dataFE())

      sliderInput(ns("min_cells"), "Minimum cells treshold:",
        min = 0,
        max = 500,
        value = 10,
        step = 1
      )
    })


    output$o_min_samples <- renderUI({
      req(dataFE())
      n_sample <- ncol(dataFE()$SE_abundance)
      sliderInput(ns("min_samples"), "Minimum samples:",
        min = 0,
        max = n_sample,
        value = round(n_sample / 4),
        step = 1
      )
    })


    output$conditions_ <- renderUI({
      req(dataFE())
      pickerInput(
        inputId = ns("selected_conditions"), "Condition:", choices = colnames(meta_names()), selected = NULL,
        multiple = TRUE,
        options = pickerOptions(maxOptions = 1)
      ) |>
        shinyInput_label_embed(
          shiny_iconlink() |>
            bs_embed_popover(
              title = "Column with the condition of interest, conditions parameter in yaml header", content = "Choose a column name with 2 parameters", placement = "left"
            )
        )
    })

    output$batch_ <- renderUI({
      req(dataFE())

      pickerInput(
        inputId = ns("selected_batch"), "Batch Column:", choices = colnames(meta_names()), selected = NULL,
        multiple = TRUE,
        options = pickerOptions(maxOptions = 1)
      ) |>
        shinyInput_label_embed(
          shiny_iconlink() |>
            bs_embed_popover(
              title = "Column with the possible Batch parameter, batch parameter in yaml header", content = "Choose a column name with a batch parameter", placement = "left"
            )
        )
    })


    output$annotation_metacluster <- renderUI({
      req(dataFE())
      req(row_data())
      pickerInput(
        inputId = ns("selected_annotation"), "Annotation Column:", choices = row_data() |> colnames(),
        selected = NULL,
        multiple = TRUE,
        options = pickerOptions(maxOptions = 1)
      ) |>
        shinyInput_label_embed(
          shiny_iconlink() |>
            bs_embed_popover(
              title = "Column with the possible Annotation parameter, annotation_column parameter in yaml header", content = "Choose a column name with an annotation reference", placement = "left"
            )
        )
    })

    # output$cluster_id_ <- renderUI({
    #   req(dataFE())
    #
    #   pickerInput(
    #     inputId = ns("selected_cluster_id_"), "Cluster ref Column:", choices = row_data() |> colnames(),
    #     selected = 1,
    #     multiple = FALSE
    #   ) |>
    #     shinyInput_label_embed(
    #       shiny_iconlink() |>
    #         bs_embed_popover(
    #           title = "Column with the cluster reference, cluster_id parameter in yaml header", content = "Choose the column with clustering reference", placement = "left"
    #         )
    #     )
    # })



    output$annot_on_plots_ <- renderUI({
      req(dataFE())

      pickerInput(
        inputId = ns("selected_annot_on_plots"), "Meta annotation:", choices = colnames(meta_names()), selected = NULL,
        multiple = TRUE
      ) |>
        shinyInput_label_embed(
          shiny_iconlink() |>
            bs_embed_popover(
              title = "Columns to use for the graphics annotation, plot_legends parameter in yaml header", content = "Choose columns to annotate the figures", placement = "left"
            )
        )
    })

    selected_values_string <- reactiveVal(NULL)
    prev_selected <- reactiveVal(NULL)



    observe({
      # req(input$selected_annot_on_plots)
      if (!identical(input$selected_annot_on_plots, prev_selected())) {
        prev_selected(input$selected_annot_on_plots)

        if (length(input$selected_annot_on_plots) == 0) {
          selected_values_string(NULL)
        } else {
          selected_values_string(input$selected_annot_on_plots)
        }
      }
    })

    # Display the selected_values_string
    output$selected_values_output <- renderPrint({
      selected_values_string()
    })

    observeEvent(
      input$run_qc,
      ignoreInit = TRUE,
      {
        # P_num <- as.numeric(gsub("p|0", "", r_global$project_folder_name))
        # Create a temporary directory
        new_step(
          project = r_global$project_id,
          label = "mod_qc", output_type = "html", input_hash_tag = r_global$dataFE
        ) -> new_step


        step_row <- tail(new_step, n = 1)
        title <- projects(r_global$project_id) |> dplyr::pull(title)
        # Create the sensory report
        res_info_report <- withProgress(message = "Rendering report...", value = 0, {
          
          create_report(
            # list_param_report = list(
            #   path =,
            #   rds =
            #   min_cells = 100
            # ),
            list_param_report = list(
              adcode = input$codex,
              analyse_type = input$analyse_type,
              annotation_column = input$selected_annotation,
              batch = input$selected_batch,
              # cluster_id = input$selected_cluster_id_,
              cluster_id = "cluster_id",
              # col_block= "patient_id",
              # cols_design= !expr c( "cond","acqdate"),
              # cols_design_GLMM_fixed= !expr c( "cond"),
              # cols_design_GLMM_random= !expr c("acqdate"),
              conditions = input$selected_conditions,
              # fc = as.double(input$choose_fc),
              # individuals = input$selected_individuals,
              log_bar = input$logex,
              min_cells = input$min_cells,
              # min_samples = input$min_samples,
              path = r_global$path_to_data, # "/home/lohmann/analyticyte_first"
              # p_value = input$given_pvalue,
              plot_legends = selected_values_string(),
              project = {
                if (!is.na(title)) title else ""
              },
              rds = "dataFE.rds",
              show_parameters = input$show_param_yaml,
              show_summary = input$show_summary_data,
              sorted = input$sortex
              # subset_column= input$,
              # transform= false,
            ),
            output_dir = paste(projects_folder(), r_global$project_folder_name, sep = "/"), # expect projects/p0001
            hash_tag = step_row$hash_tag, # expect an hash_tag
            format = "html",
            extension = "crcm",
            o_report = NULL
          )
        })
        showNotification("HTML created", type = "message")
        reloadTrigger(reloadTrigger() + 1)
        finish_step(project = r_global$project_id, SID = step_row$SID)

        # Save the info about the report
        r_global$report_dir <- res_info_report$output_dir_report
        r_global$report_file <- res_info_report$output_file_report
        r_global$yaml_file <- res_info_report$output_yaml
      }
    )


    # mod_report_server("mod_report", r_global = r_global)
  })
}

## To be copied in the UI
# mod_qc_ui("qc_1")

## To be copied in the server
# mod_qc_server("qc_1")



#' report UI Function
#'
#' @description A shiny Module. taken from https://github.com/ThinkR-open/sensoreport , Margot Brard work
#'
#' @param id,input,output,session Internal parameters for {shiny}.
#'
#' @noRd
#'
#' @importFrom shiny NS tagList
mod_report_ui <- function(id) {
  ns <- NS(id)

  tagList(
    tags$div(
      class = "col-xs-12 col-sm-8",
      tags$div(
        htmlOutput(ns("report"))
      )
    )
  )
}


#' report Server Functions taken from https://github.com/ThinkR-open/sensoreport , Margot Brard work
#'
#' @noRd
mod_report_server <- function(id, r_global) {
  moduleServer(id, function(input, output, session) {
    ns <- session$ns

    # Preview the report
    observeEvent(
      c(
        r_global$report_dir,
        r_global$report_file
      ),
      {
        add_resource_path(
          "reportlibrary", dirname(r_global$report_dir),
        )

        output$report <- renderUI({
          tagList(
            tags$div(
              style = "padding-top: 1em;",
              tags$iframe(
                seamless = "seamless",
                src = paste0("reportlibrary/", r_global$report_file),
                frameborder = "0",
                style = "width:130%;height:110vh;" # Adjusted values for width and height              )
              )
            )
          )
        })
      }
    )
  })
}
