#' table UI Function
#'
#' @description A shiny Module.
#'
#' @param id,input,output,session Internal parameters for {shiny}.
#'
#' @noRd
#'
#' @importFrom shiny NS tagList
mod_table_ui <- function(id, show_add_button = TRUE) {
  ns <- NS(id)

  btn_add <- if (show_add_button) {
    fluidRow(
      column(
        width = 2,
        actionButton(
          ns("add"),
          "Add",
          class = "btn-success",
          style = "color: #fff;",
          icon = icon("plus"),
          width = "100%"
        ),
        tags$br(),
        tags$br()
      )
    )
  } else {
    NULL
  }

  tagList(
    btn_add,
    fluidRow(
      column(
        width = 12,
        title = "Element to store",
        DT::DTOutput(ns("table_tb")),
        tags$br(),
        tags$br()
      )
    ),
    tags$script(src = "custom.js"),
    tags$script(paste0("table_module_js('", ns(""), "')"))
  )
}




#' table Server Functions
#'
#' @noRd
mod_table_server <- function(id, table, what, ID, l=10) {
  moduleServer(id, function(input, output, session) {
    ns <- session$ns
    # fct <- match.fun(what)
    out <- table

    ids <- out[[ID]]

    # actions <- purrr::map_chr(ids, function(id_) {
    #   paste0(
    #     '<div class="btn-group" style="width: 75px;" role="group" aria-label="Basic example">
    #       <button class="btn btn-primary btn-sm edit_btn" data-toggle="tooltip" data-placement="top" title="Edit" id = ', what, "_table_", id_, "-edit", ' style="margin: 0"><i class="fa fa-pencil-square-o"></i></button>
    #       <button class="btn btn-danger btn-sm delete_btn" data-toggle="tooltip" data-placement="top" title="Delete" id = ', what, "_table_", id_, "-delete", ' style="margin: 0"><i class="fa fa-trash-o"></i></button>
    #     </div>'
    #   )
    # })

    actions <- sapply(ids, function(id_) {
      paste0(
        '<div class="btn-group" style="width: 75px;" role="group" aria-label="Basic example">
      <button class="btn btn-primary btn-sm edit_btn" data-toggle="tooltip" data-placement="top" title="Edit" id = ', what, "_table_", id_, "-edit", ' style="margin: 0"><i class="fa fa-pencil-square-o"></i></button>
      <button class="btn btn-danger  btn-sm delete_btn" data-toggle="tooltip" data-placement="top" title="Delete" id = ', what, "_table_", id_, "-delete", ' style="margin: 0"><i class="fa fa-trash-o"></i></button>
    </div>'
      )
    }, USE.NAMES = FALSE)


    # Set the Action Buttons row to the first column of the `mtcars` table
    out <- cbind(
      dplyr::tibble(" " = actions),
      out
    )


    output$table_tb <- DT::renderDT({
      prettyDT(l=l,
        x = out, rownames = FALSE,
        drawCallback = DT::JS("function(settings) {
                   // removes any lingering tooltips
                   $('.tooltip').remove()
                 }")
      )
    })
    print(what)
  })
}

## To be copied in the UI
# mod_table_ui("table_1")

## To be copied in the server
# mod_table_server("table_1")
