mfi_matrix <- function(
    mfis_table=NULL, 
    applied_fct="mean",
    marker_id=NULL,
    marker_and_file_pos=c(1,2)


){

  mfis_table %>%
    tidyr::pivot_longer(
      cols = colnames(mfis_table)[-marker_and_file_pos], # two first columns are espected to be file id and marker id
      names_to = "cluster",
      values_to = "mfi_intensity"
    ) %>%
    dplyr::mutate(mfi_intensity = as.numeric(mfi_intensity)) -> mfis_long

  checkmate::assertChoice(applied_fct, c("mean", "median"))
  fct <- match.fun(applied_fct)

  mfis_long %>%
    stats::na.omit() %>%
    dplyr::group_by(!!rlang::sym(marker_id), cluster) %>% # cluster created above
    dplyr::summarise(median_mfi = fct(mfi_intensity)) -> merged_mfis # mfi_intensity created above

  merged_mfis %>%
    tidyr::pivot_wider(names_from = cluster, values_from = median_mfi) |>
    as.data.frame() |>
    tibble::column_to_rownames(marker_id) -> heatmap_merged_mfis


  apply(heatmap_merged_mfis, 1, function(row) {
    norm <- row / max(row)
    norm[norm <= 0] <- 0
    norm
  }) |> t() -> heatmap_scaled_merged_mfis

    return(heatmap_scaled_merged_mfis)
}